function varargout = ORBIT(varargin)
% "Optical Reconstruction of the Beta Ionization Track" (ORBIT) [1] is an
% image reconstruction software for radioluminescence microscopy 
% 
% 
% Version 1.11
%
% Inputs -
%   H value: Cleans up background noise. Start from 0 and increase until
%           basckground noise is suppressed but scintillation flashes
%           remain.
%   Binary Threshold: Removes any signals lower than the set value. Adjust
%           the number until the scintillation flashes are clearly
%           segmented.
%   Track Length: Removes scintillation tracks that are lower than set 
%           value. Approximate acceptable length can be determined by 
%           toggling between different frames. Default = 13
%   Track Sharpness: Removes any scintillation tracks that are blurrier 
%           than set value. Use this value to remove out of focus or low 
%           energy tracks that are close to the background. Default = 20
%
% 'Evaluation mode' - Press 'Refresh' after filling out initial input
%  values. '<<' will switch to the previous frame and '>>' to the next one. 
%
% 'Reconstruct' - Press to initiate reconstruction process. The
% progress can be tracked in the Command Window. ORBIT uses parallel
% processing, when available, thus the frames are not processed in 
% sequential order.
%
% Reference:
% [1] Pratx G, Chen K, Sun C, Axente M, Sasportas L, Carpenter C & Xing L, 
% "High-Resolution Radioluminescence Microscopy of FDG Uptake by 
% Reconstructing the Beta Ionization Track", J. Nucl. Imag. 54(10) 
% pp. 1841-1846, Oct. 2013
%
% Credits: Tae Jin Kim & Guillem Pratx, Stanford University

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @ORBIT_OpeningFcn, ...
                   'gui_OutputFcn',  @ORBIT_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before ORBIT is made visible.
function ORBIT_OpeningFcn(hObject, ~, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to ORBIT (see VARARGIN)

% Choose default command line output for ORBIT
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

global JJ
global st
global W
global exmax
global HT_val
global mask_em_val
global ind
global mag
global GauPar
global itp_tau
global itp_yld
global pixsize
global focal %focal length of objective lens
global exptime
global LLMtube
global ImQuery

movegui('north');

JJ=[]; %Initialized as blank BF image
LLMtube=36; %Change focal length depending on LLM tube lens

ImQuery=0;

exmax=10; %set default values
st=20;
HT_val=100;
mask_em_val=3000;
W=10;
ind=1;
mag=20;
GauPar=0.6;

itp_tau=109.728/60;
itp_yld=0.9686;

pixsize=0;
exptime=0;

focal=180/20;

set(handles.text8, 'String', '');

set(handles.edit1, 'String', '100');
set(handles.edit2, 'String', '3000');
set(handles.edit3, 'String', '10');
set(handles.edit4, 'String', '20');

set(handles.text1, 'String', '');
set(handles.text2, 'String', '');
set(handles.text3, 'String', '');
    
axes(handles.axes1); cla; axis image; axis off; 
axes(handles.axes2); cla; axis image; axis off; 
axes(handles.axes3); cla; axis image; axis off; 
axes(handles.axes4); cla; axis image; axis off;
title('');

function varargout = ORBIT_OutputFcn(~, ~, handles) 
varargout{1} = handles.output;


function pushbutton1_Callback(~, ~, handles)

global JJ
global BF_xsize
global BF_ysize
global ind
global ImQuery

ImQuery=0; %reset ImQuery to default when reopening BF image

JJ=[]; %initialize JJ when reopening image file, including when cancelling from file select screen
BF_ysize=[];
BF_xsize=[];

[filename, pathname]= uigetfile({'*.TIF;*.tif','TIF image (*.tif or *.TIF)'},'Multiselect','off');

raw=[pathname, filename];

if numel(raw)>2
    
    [BF_ysize, BF_xsize]=size(imread(raw));
    JJ = (double(imread(raw)));

    axes(handles.axes4); 
    imagesc(JJ,[0.75*mean(mean(JJ)),1.75*mean(mean(JJ))]);
    title('Brightfield image');

    axis image;
    axis off;
    colormap gray;

end

ind=1; %Reset current Track # to 1


function pushbutton2_Callback(~, ~, handles)

global JJ
global ImQuery

if numel(JJ)==0 %If no BF file is initially selected
    msgbox('Please select brightfield image first')
else
    
set(handles.text8, 'String', 'Busy');
pause(0.001); %pause for 1ms to allow time to print 'Busy'

BB = imnorm(JJ);

f=figure;
set(f, 'Position', [200, 200, 800, 800]);
movegui(f,'onscreen');
imagesc(BB,[0.75*mean(BB(:)),1.75*max(BB(:))]); %display only parameters
axis image; axis off; colormap gray;

%create question dialog to ask if the current brightfield should be
%replaced by intensity-normalized image
choice = questdlg('Would you like to keep the image?', ...
	'Image quality', 'Yes','No','No');
% Handle response
switch choice
    case 'Yes'        
        close(f)
        ImQuery=1;
        JJ = BB;
        imagesc(JJ,[0.75*mean(mean(JJ)),1.75*mean(mean(JJ))]);
        axes(handles.axes4);
        title('Brightfield image');
        axis image; axis off;colormap gray;
        
    case 'No'
        close(f)
        ImQuery=0;
        JJ=double(JJ);
        JJ=JJ./max(max(JJ));
        imagesc(JJ,[0.75*mean(mean(JJ)),1.75*mean(mean(JJ))]);
        axes(handles.axes4);
        title('Brightfield image');
        axis image; axis off;colormap gray;
end

set(handles.text8, 'String', 'Done')
end


function pushbutton3_Callback(~, ~, handles)

global JJ
global D2
global numIm
global bitdepth
global BF_xsize
global vx
global vy
global vxd
global vyd
global X
global Y
global sc
global ind
global pathname
global files

if numel(JJ)==0 %If no BF file is initially selected
    msgbox('Please select brightfield image first')
else

pathname=[]; %reinitialize pathname/filename when reopening scintillation images
files=[];
D2=[];
X=[];
Y=[];
bitdepth=[];
numIm=[];
files=[];
sc=[];
vx=[];
vxd=[];
vy=[];
vyd=[];

pathname=uigetdir();

if numel(pathname)>1 %if the file is actually selected
    
    set(handles.text8, 'String', 'Busy');
    pause(0.001);

    files=dir([pathname,filesep,'*.tif']);
    
    if isempty(files)==1
        msgbox('No image files in folder')
    else
    fnum=numel(files);

    info=imfinfo([pathname,filesep,files(1).name]);
    numIm=numel(info);

    numIm(1)=numel(info);
    bitdepth(1)=info.BitDepth;
    imwidth(1)=info.Width;
    imheight(1)=info.Height;

    if fnum>1 %if multiple files exist in folder
    for i=2:fnum
    info=imfinfo([pathname,filesep,files(i).name]);
    numIm(i,1)=numel(info);
    bitdepth(i,1)=info.BitDepth;
    imwidth(i,1)=info.Width;
    imheight(i,1)=info.Height;
    end
    end

if all(bitdepth==bitdepth(1)) && all(imwidth==imwidth(1)) && all(imheight==imheight(1)) %check if all files have the same bitdepth, width and height

        LUMraw=[pathname,filesep,files(1).name];

        [vx, vy] = size(imread(LUMraw));
        [X,Y] = meshgrid(1:vx,1:vy);

        sc=BF_xsize/vx; %Binning scale factor

        if exist('dark.mat','file')
        load dark.mat
            if exist('D_std','var')==0 %D_std variable is formed if dark.mat is created using actual dark images
                msgbox('NOTE: dark.mat file is based on approximate dark image values')
            end
            
        D2 = D;
        vxd=size(D2,2); %CHECK DIMENSION
        vyd=size(D2,1);        
            if vx~=vxd && vy~=vyd
            msgbox('WARNING: Dark image size different from scintillation image')
            end
        else

            choice = questdlg('No dark.mat file! Do you have a dark image?', ...
            'Image quality', 'Yes','No','No');
            % Handle response
            switch choice
            case 'Yes'
            [filename2, pathname2]= uigetfile({'*.TIF'},'Multiselect','on');

            query=iscell(filename2); %Query if multiple images are selected or not
            numIm2=0; %Initialize numIm2 to supress error
            
                if query==0 %if dark image is multitiff

                dark=[pathname2,filename2];
                
                if dark==0

                else
                numIm2=numel(imfinfo(dark));
                Temp=zeros(numIm2,1);
                [vyd, vxd] = size(imread(dark));

                    if vx==vxd && vy==vyd

                    D=zeros(vxd,vyd);
                    D_std=zeros(vxd,vyd);

                    %stack all dark images into cell
                        for i=1:numIm2
                        Current=double(imread(dark,i));
                        B{i,1}=Current;
                        end

                        for j=1:vyd
                            for k=1:vxd
                                for i=1:numIm2
                                Temp(i,1)=B{i,1}(j,k); %Arrange dark pixel information in a given coordinate in a single column
                                end
                            D(j,k)=mean(Temp);
                            D_std(j,k)=std(Temp);
                            end
                        end

                        save dark.mat D D_std
                        else
                        msgbox('Please check files and reopen: dark image size different from scintillation image')
                    end
                end

                else %if dark image is a stack of single tiff images

                    
                dark=imread([pathname2,char(filename2(1))]);

                numIm2=numel(filename2);
                Temp=zeros(numIm2,1);
                [vxd, vyd] = size(dark);

                if vx==vxd && vy==vyd

                D=zeros(vxd,vyd);
                D_std=zeros(vxd,vyd);

                %stack all dark images into cell
                    for i=1:numIm2
                        Current=double(imread([pathname2,char(filename2(i))]));
                        B{i,1}=Current;
                    end

                    for j=1:vyd
                         for k=1:vxd
                             for i=1:numIm2
                                 Temp(i,1)=B{i,1}(j,k); %Arrange dark pixel information in a given coordinate in a single column
                             end
                             D(j,k)=mean(Temp);
                             D_std(j,k)=std(Temp);
                         end
                     end

                save dark.mat D D_std
                else
                    msgbox('Please check files and reopen: dark image size different from scintillation image')

                end
                end

            case 'No'
                
                vxd=vx;                
                vyd=vy;
                D=ones(vxd,vyd);
                
                if fnum>1
                darktemp=zeros(round(numel(numIm)/10),1); %only 10% of entire scintillation images used to reduce computation time

                    for i=1:round(numel(numIm)/10)
                    darktemp(i,1)=(mean(mean(imread([pathname,filesep,files(i).name])))+min(min(imread([pathname,filesep,files(i).name]))))./2; %average of mean and min scintillation images yield similar values to actual dark image
                    end
                    
                    
                else

                darktemp=zeros(round(numIm/10),1); 

                    for i=1:round(numIm/10)
                    darktemp(i,1)=(mean(mean(imread(LUMraw,i)))+min(min(imread(LUMraw,i))))./2; 
                    end
                end

                D=D.*mean(darktemp);
                save dark.mat D                   
               
            end
        end

        else
            if all(bitdepth==bitdepth(1)) 
                a='Image bit depth consistency = GOOD';
            else
                a='Image bit depth consistency = ERROR(Please Check)';
            end
            if all(imwidth==imwidth(1))
                b='Image Width consistency = GOOD';
            else
                b='Image Width consistency = ERROR(Please Check)';
            end
            if all(imheight==imheight(1))
                c='Image Height consistency = GOOD';
            else
                c='Image Height consistency = ERROR(Please Check)';
            end

        msgbox({a,b,c},'WARNING');
end

D2 = D;

set(handles.text8, 'String', 'Done')

    end
end

ind=1; %reset to first scintillation image 
end


function pushbutton7_Callback(~, ~, handles)

global JJ
global D2
global numIm
global exmax
global HT_val
global st
global mask_em_val
global sc
global vx
global vy
global vxd
global vyd
global H
global W
global X
global Y
global J
global bitdepth
global ImQuery
global pathname
global files
global GauPar

if or(isempty(files)==1,isempty(JJ)==1)
    msgbox('Please open image files')
else
    
if JJ>1 %if 'Normalize Brigthness' tab has not been pressed
    JJ=JJ./max(max(JJ));
end

if vx~=vxd && vy~=vyd
    set(handles.text8, 'String', '')
    msgbox('WARNING: Dark image size different from scintillation image')
else
    
    set(handles.text8, 'String', 'Busy')
    pause(0.001);

    matlabpool open
    if matlabpool('size')>1
    else
        matlabpool close;
    end

S = zeros(vx,vy); % initialize image

tic

if numel(numIm)>1 %if there are multiple files

    if sum(numIm)/numel(numIm)>1 %if there is a multitiff in the image files

        for ii=1:numel(numIm)

            LUMraw=[pathname,filesep,files(ii).name];

        parfor i=1:numIm(ii)

            disp(['Image # ' int2str(i)]); 
            J = double(imread(LUMraw,i));            
            [K, bw_label] = trklbl(J,HT_val,mask_em_val,GauPar);

            % Track processing
            for k = 1:max(bw_label(:))
                
                L = (K-D2).*imdilate(bw_label==k, strel('disk',1));
                L2 = (J-D2).*imdilate(bw_label==k, strel('disk',1));
                [veto,j] = max(L2(:));
                x = floor((j-1)/vx)+1;
                y = (j-1)-vx*(x-1)+1;

                    if (veto<0.75*(2^bitdepth(1)))

                                        % Track extent
                        extt = extrack(L,x,y,W);
                                        % Fourier analysis
                        Z1 = sum(abs(fft(L,[],1)),2);
                        Z2 = sum(abs(fft(L,[],2)),1);

                        z1 = find(Z1<(max(Z1(:))+min(Z1(:)))/2, 1, 'first');
                        z2 = find(Z2<(max(Z2(:))+min(Z2(:)))/2, 1, 'first');

                    if (max(z1,z2)>st && extt<=exmax)
                        [px,py]=centmass(L,x,y,W);
                        xr = round(px);
                        yr = round(py);
                        S = S + ( (X==xr) & (Y==yr) );
                    end
                    end
            end            
            hold off;
        end
        end

    else %if only single-tif files
        
    parfor ii=1:numel(numIm)

            disp(['Image # ' int2str(ii)]); 
            LUMraw=[pathname,filesep,files(ii).name];
            J = double(imread(LUMraw));            
            [K, bw_label] = trklbl(J,HT_val,mask_em_val,GauPar);

                        % Track processing
            for k = 1:max(bw_label(:))
                
                L = (K-D2).*imdilate(bw_label==k, strel('disk',1));
                L2 = (J-D2).*imdilate(bw_label==k, strel('disk',1));
                [veto,j] = max(L2(:));
                x = floor((j-1)/vx)+1;
                y = (j-1)-vx*(x-1)+1;

                    if (veto<0.75*(2^bitdepth(1)))

                                        % Track extent
                        extt = extrack(L,x,y,W);
                                        % Fourier analysis
                        Z1 = sum(abs(fft(L,[],1)),2);
                        Z2 = sum(abs(fft(L,[],2)),1);

                        z1 = find(Z1<(max(Z1(:))+min(Z1(:)))/2, 1, 'first');
                        z2 = find(Z2<(max(Z2(:))+min(Z2(:)))/2, 1, 'first');

                    if (max(z1,z2)>st && extt<=exmax)
                        [px,py]=centmass(L,x,y,W);
                        xr = round(px);
                        yr = round(py);
                        S = S + ( (X==xr) & (Y==yr) );
                    end
                    end
            end
            hold off;
    end
    end
      
else %if there is only a single file
    if sum(numIm)/numel(numIm)>1 %if there is a multitiff in the image file
        
        LUMraw=[pathname,filesep,files.name];
        
        parfor ii=1:numIm

            disp(['Image # ' int2str(ii)]); 
            J = double(imread(LUMraw,ii));            
            [K, bw_label] = trklbl(J,HT_val,mask_em_val,GauPar);

                        % Track processing
            for k = 1:max(bw_label(:))
                
                L = (K-D2).*imdilate(bw_label==k, strel('disk',1));
                L2 = (J-D2).*imdilate(bw_label==k, strel('disk',1));
                [veto,j] = max(L2(:));
                x = floor((j-1)/vx)+1;
                y = (j-1)-vx*(x-1)+1;

                    if (veto<0.75*(2^bitdepth(1)))

                                        % Track extent
                        extt = extrack(L,x,y,W);
                                        % Fourier analysis
                        Z1 = sum(abs(fft(L,[],1)),2);
                        Z2 = sum(abs(fft(L,[],2)),1);

                        z1 = find(Z1<(max(Z1(:))+min(Z1(:)))/2, 1, 'first');
                        z2 = find(Z2<(max(Z2(:))+min(Z2(:)))/2, 1, 'first');

                    if (max(z1,z2)>st && extt<=exmax)
                        [px,py]=centmass(L,x,y,W);
                        xr = round(px);
                        yr = round(py);
                        S = S + ( (X==xr) & (Y==yr) );
                    end
                    end
            end

            hold off;
        end
    end
end

matlabpool close;

toc

figure;
fsz = 20; %font size

SS = imresize(imfilter(S,fspecial('Gaussian',[21 21],GauPar/3)),sc,'nearest'); % (low-pass) gaussian filter on the large (x4) spot image. WHY? %CHANGE MAG HERE!
cmap = mycmap(0.08,0.10);
imagesc(SS);axis image;axis off;colormap(cmap);cc=colorbar;
set(cc,'FontSize',fsz,'FontName','Helvetica');
set(get(cc,'ylabel'),'string','counts/px','fontsize',fsz,'FontName','Helvetica');

SSS = SS/max(SS(:));
imwrite(64*SSS, cmap,'DecayMap.png'); 
imwrite(64*SSS, 1-gray,'DecayMap_BW.png'); 

if ImQuery==1
    JJJ = (JJ-0.9) / (1.1-0.9); %Can be modified if brightfield image is not satisfactory
    JJJ(JJJ>1) = 1; JJJ(JJJ<0) = 0;
    imwrite(JJ./max(max(JJ)).*2^4, gray,'BF.png'); 

elseif ImQuery==0
    JJJ=JJ/max(JJ(:));
    imwrite(JJJ.*2^4, gray,'BF.png'); 
end

figure;
H = overlim2(JJJ,SSS,0.1,1,0.1,1,0.01, jet);
imagesc(H); axis image;axis off;
imwrite(H, 'Fusion.png');

Data_DecayRaw=S; %Raw decay data
Img_BF=JJJ; %Brightfield image
Img_DecayRawSmoothed=SS; %Gaussian smoothed decay image
Img_DecayProcessed=SSS; %for overlaying with brightfield image
Img_Fusion=H; %Brightfield overlayed with gaussian smoothed decay image
Par_TotalImgNumber=sum(numIm(:)); %Number of scintillation images
Par_Binning=sc; %Binning value for scintillation image
Par_TrackLength=exmax; %Track length rejection parameter
Par_Sharpness=st; %Sharpness rejection parameter
Par_H_Value=HT_val;
Par_BinaryThreshold=mask_em_val;

save ScintData.mat  Data_DecayRaw Img_BF Img_DecayRawSmoothed Img_DecayProcessed Img_Fusion Par_TotalImgNumber Par_Binning Par_TrackLength Par_Sharpness Par_H_Value Par_BinaryThreshold;
set(handles.text8, 'String', 'Done')
clearvars;

end
end


function pushbutton4_Callback(~, ~, handles)

global JJ
global D2
global bw_label
global sc
global st
global exmax
global HT_val
global mask_em_val
global K
global J
global HT
global H
global W
global vx
global vy
global vxd
global vyd
global X
global Y
global ind
global bitdepth
global ImQuery
global numIm
global pathname
global files
global GauPar

if ind<=1;
    ind=1;
else
    ind=ind-1;
end

if or(isempty(files)==1,isempty(JJ)==1)
    msgbox('Please open image files')
else
    
if JJ>1 %if 'Normalize Brigthness' tab has not been pressed
    JJ=JJ./max(max(JJ));
end

if vx~=vxd && vy~=vyd
    set(handles.text8, 'String', '')
    msgbox('WARNING: Dark image size different from scintillation image')
else
    
if sum(numIm)/numel(numIm)>1 %if there is a multitiff in the image files
       
    LUMraw=[pathname,filesep,files(1).name];
    J = double(imread(LUMraw,ind));
    [H,K,HT,bw_label]=track1(JJ,J,HT_val,mask_em_val,ImQuery,sc,GauPar);
        
        axes(handles.axes1);imagesc(K);axis image;axis off;colormap gray;
        axes(handles.axes2);imagesc(J);axis image;axis off;colormap gray;
        axes(handles.axes3);imagesc(HT);axis image;axis off;colormap gray;
        axes(handles.axes4);imagesc(H,[0.75*mean(mean(JJ)),1.75*mean(mean(JJ))]);axis image;axis off;colormap gray;
        title(['Frame# = ' int2str(ind)]);
        
    % Track processing
    track2(J,K,D2,vx,X,Y,bw_label,bitdepth,W,exmax,sc,st);
    
else %if no multitiff files present
    
    LUMraw=[pathname,filesep,files(ind).name];
    J = double(imread(LUMraw));
    [H,K,HT,bw_label]=track1(JJ,J,HT_val,mask_em_val,ImQuery,sc,GauPar);
        
        axes(handles.axes1);imagesc(K);axis image;axis off;colormap gray;
        axes(handles.axes2);imagesc(J);axis image;axis off;colormap gray;
        axes(handles.axes3);imagesc(HT);axis image;axis off;colormap gray;
        axes(handles.axes4);imagesc(H,[0.75*mean(mean(JJ)),1.75*mean(mean(JJ))]);axis image;axis off;colormap gray;
        title(['Frame# = ' int2str(ind)]);
    
    % Track processing
    track2(J,K,D2,vx,X,Y,bw_label,bitdepth,W,exmax,sc,st);
    
end
    hold off;
end
end

if isempty(pathname)==0;  
impixelinfo;
end


function pushbutton5_Callback(~, ~, handles)

global JJ
global D2
global bw_label
global HT_val
global mask_em_val
global sc
global st
global exmax
global HT
global vx
global vy
global vxd
global vyd
global W
global X
global Y
global K
global J
global H
global ind
global bitdepth
global ImQuery
global numIm
global pathname
global files
global GauPar

if or(isempty(files)==1,isempty(JJ)==1)
    msgbox('Please open image files')
else
    
if vx~=vxd && vy~=vyd
    set(handles.text8, 'String', '')
    msgbox('WARNING: Dark image size different from scintillation image')
else

if JJ>1 %if 'Normalize Brigthness' tab has not been pressed
    JJ=JJ./max(max(JJ));
end

    set(handles.text1, 'String', 'Raw Scintillation Image');
    set(handles.text2, 'String', 'Low Intensity Suppressed');
    set(handles.text3, 'String', 'Gaussian Smoothed');
    
if sum(numIm)/numel(numIm)>1 %if there is a multitiff in the image files
       
    LUMraw=[pathname,filesep,files(1).name];
    J = double(imread(LUMraw,ind));
    [H,K,HT,bw_label]=track1(JJ,J,HT_val,mask_em_val,ImQuery,sc,GauPar);
        
        axes(handles.axes1);imagesc(K);axis image;axis off;colormap gray;
        axes(handles.axes2);imagesc(J);axis image;axis off;colormap gray;
        axes(handles.axes3);imagesc(HT);axis image;axis off;colormap gray;
        axes(handles.axes4);imagesc(H,[0.75*mean(mean(JJ)),1.75*mean(mean(JJ))]);axis image;axis off;colormap gray;
        title(['Frame# = ' int2str(ind)]);
    
    % Track processing
    track2(J,K,D2,vx,X,Y,bw_label,bitdepth,W,exmax,sc,st);
    
else
    
    LUMraw=[pathname,filesep,files(ind).name];
    J = double(imread(LUMraw));
    [H,K,HT,bw_label]=track1(JJ,J,HT_val,mask_em_val,ImQuery,sc,GauPar);
        
        axes(handles.axes1);imagesc(K);axis image;axis off;colormap gray;
        axes(handles.axes2);imagesc(J);axis image;axis off;colormap gray;
        axes(handles.axes3);imagesc(HT);axis image;axis off;colormap gray;
        axes(handles.axes4);imagesc(H,[0.75*mean(mean(JJ)),1.75*mean(mean(JJ))]);axis image;axis off;colormap gray;
        title(['Frame# = ' int2str(ind)]);
    
    % Track processing
    track2(J,K,D2,vx,X,Y,bw_label,bitdepth,W,exmax,sc,st);
    
end        
    hold off;
end
end

if isempty(pathname)==0;  
impixelinfo;
end


function pushbutton6_Callback(~, ~, handles)

global JJ
global bw_label
global D2
global numIm
global sc
global st
global exmax
global HT_val
global mask_em_val
global K
global J
global HT
global H
global W
global vx
global vy
global vxd
global vyd
global X
global Y
global ind
global bitdepth
global ImQuery
global pathname
global files
global GauPar

if or(isempty(files)==1,isempty(JJ)==1)
    msgbox('Please open image files')
else
    
if vx~=vxd && vy~=vyd
    set(handles.text8, 'String', '')
    msgbox('WARNING: Dark image size different from scintillation image')
else

if JJ>1 %if 'Normalize Brigthness' tab has not been pressed
    JJ=JJ./max(max(JJ));
end
    
if sum(numIm)/numel(numIm)>1 %if there is a multitiff in the image files

    if ind>=numIm(1);
    ind=numIm(1);
    else
    ind=ind+1;
    end
    
    LUMraw=[pathname,filesep,files(1).name];
    J = double(imread(LUMraw,ind));
    [H,K,HT,bw_label]=track1(JJ,J,HT_val,mask_em_val,ImQuery,sc,GauPar);

        axes(handles.axes1);imagesc(K);axis image;axis off;colormap gray;
        axes(handles.axes2);imagesc(J);axis image;axis off;colormap gray;
        axes(handles.axes3);imagesc(HT);axis image;axis off;colormap gray;
        axes(handles.axes4);imagesc(H,[0.75*mean(mean(JJ)),1.75*mean(mean(JJ))]);axis image;axis off;colormap gray;
        title(['Frame# = ' int2str(ind)]);
    
    track2(J,K,D2,vx,X,Y,bw_label,bitdepth,W,exmax,sc,st);
    
else
    
    if ind>=numel(numIm);
    ind=numel(numIm);
    else
    ind=ind+1;
    end
    
    LUMraw=[pathname,filesep,files(ind).name];
    J = double(imread(LUMraw));
    [H,K,HT,bw_label]=track1(JJ,J,HT_val,mask_em_val,ImQuery,sc,GauPar);
    
        axes(handles.axes1);imagesc(K);axis image;axis off;colormap gray;
        axes(handles.axes2);imagesc(J);axis image;axis off;colormap gray;
        axes(handles.axes3);imagesc(HT);axis image;axis off;colormap gray;
        axes(handles.axes4);imagesc(H,[0.75*mean(mean(JJ)),1.75*mean(mean(JJ))]);axis image;axis off;colormap gray;
        title(['Frame# = ' int2str(ind)]);
    
    % Track processing
    track2(J,K,D2,vx,X,Y,bw_label,bitdepth,W,exmax,sc,st);
end
    hold off;
end
end

if isempty(pathname)==0;  
impixelinfo;
end


function edit3_Callback(hObject, ~, handles)

global exmax

handles.defaultEdit1String = '10';
exmax=str2double(get(hObject,'String'));

function edit3_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit4_Callback(hObject, ~, handles)

global st

handles.defaultEdit2String = '20';
st=str2double(get(hObject,'String'));

function edit4_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit1_Callback(hObject, ~, handles)

global HT_val
handles.defaultEdit3String = '100';
HT_val=str2double(get(hObject,'String'));

function edit1_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function edit2_Callback(hObject, ~, handles)

global mask_em_val

handles.defaultEdit4String = '3000';
mask_em_val=str2double(get(hObject,'String'));

function edit2_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function [J] = imnorm(P, t, s)

if (nargin==1)
    s = 10;
    t = min(P(:)) + 0.3 * ( median(P(:)) - min(P(:)) );
elseif (nargin==2)
    s = 10;
end

[vy,vx] = size(P);

J = zeros(vx,vy);
for a=1:vy
    for b=1:vx
        R = P(max(1,a-s):min(vy,a+s),max(1,b-s):min(vx,b+s));
        Rs=R(R>t);
        if (length(Rs)>75)
            J(a,b) = double(P(a,b)) / mean(Rs);
        else
            J(a,b) = 0;
        end
    end
end


function I = overlim2(A,B,miA,maA,miB,maB,sigm,cmap)

A = double(A);
B = double(B);

s=size(A);
H = zeros(prod(s),3);

if (sigm>0)
B = imfilter(B,fspecial('Gaussian',[31 31], sigm),'same');
end

b = (B(:) - miB) / (maB - miB);
a = (A(:) - miA) / (maA - miA );
b(b<0)=0;
a(a<0)=0;
b(b>1)=1;
a(a>1)=1;
a(isnan(a)) = 0;
b(isnan(b)) = 0;

t = 0;

 J = cmap;
 x = linspace(0,1,64);
 H(:,1) = 0.7*(b>t).*interp1(x,J(:,1),b) + 0.7*a;
 H(:,2) = 0.7*(b>t).*interp1(x,J(:,2),b) + 0.7*a;
 H(:,3) = 0.7*(b>t).*interp1(x,J(:,3),b) + 0.7*a;

I = reshape(H,[s(1) s(2) 3]);

I(I<0)=0;
I(I>1)=1;


function I = overlim4(A,B,C,miA,maA,miC,maC,sigm)

A = double(A);
B = double(B);

M = (sum(B,3)==0);
I = B / 255;

A = (A - miA) / (maA - miA );
C = imfilter((C - miC) / (maC - miC),fspecial('Gaussian',[21 21],sigm));

A(A<0)=0;
B(B<0)=0;
C(C<0)=0;

for i=1:3
    J = 0.6*C.*squeeze(B(:,:,i))/255 + 0.6*A;
    I(:,:,i) = J;
end

I(I<0)=0;
I(I>1)=1;


function c = mycmap(sat1,sat2)

c = zeros(64,3);

N = 64;
N1 = 1+round(sat1*N);
N5 = N-round(sat2*N);
dN = N5-N1;

N2 = N1+round(dN/8);
N3 = N1+round(7*dN/16);
N4 = N1+round(3*dN/4);

c(N1:N2-1,3) = linspace(0,1,N2-N1);
c(N2:N3-1,3) = 1;
c(N3:N4-1,3) = linspace(1,0,N4-N3);

c(N2:N3-1,2) = linspace(0,1,N3-N2);
c(N3:N4-1,2) = 1;
c(N4:N5-1,2) = linspace(1,0,N5-N4);

c(N3:N4-1,1) = linspace(0,1,N4-N3);
c(N4:64,1) = 1;


function extt = extrack(I, x, y, W)

[vx,vy] = size(I);

J=I(max(1,y-W):min(vx,y+W),max(1,x-W):min(vy,x+W));


T = 0.2*max(J(:));

J(J<0)=0;

lx = max(1,x-W)-x:min(vy,x+W)-x;
ly = max(1,y-W)-y:min(vx,y+W)-y;

[X,Y]=meshgrid(lx,ly);

xmi=min(X(J>T));
xma=max(X(J>T));

ymi=min(Y(J>T));
yma=max(Y(J>T));

extt = max([xma-xmi; yma-ymi]);


function [px,py]=centmass(I,x,y,W)

[vy,vx] = size(I); %columns/x-axis = size(I,2), rows/y-axis = size(I,1)

J=I(max(1,y-W):min(vx,y+W),max(1,x-W):min(vy,x+W));

J(J<0)=0;

lx = max(1,x-W)-x:min(vy,x+W)-x;
ly = max(1,y-W)-y:min(vx,y+W)-y;

[X,Y]=meshgrid(lx,ly);

[~,i] = max(J(:));

px = x + X(i);
py = y + Y(i);

function [H,K,HT,bw_label] = track1(JJ,J,HT_val,mask_em_val,ImQuery,sc,GauPar)

K=imfilter(J,fspecial('Gaussian',[21 21],GauPar),'replicate');
HT = imhmax(K,HT_val);  % Suppresses intensity lower than HT_val to zero
mask_em = HT>mask_em_val;  % Creates a mask where the intensity value is greater than mask_em_val
mask_em = imfill(mask_em,'holes'); %Convert to binary
mask_em = bwareaopen(mask_em, 5); %Remove objects smaller than 5 pixels from binary image
bw_label = bwlabel(mask_em,8); %Label connected components in binary image -> cells need to be sufficiently spaced

    % Live display
if ImQuery==1
    H = overlim4(JJ,label2rgb(imresize(bw_label,sc,'nearest'),'jet',[0,0,0],'shuffle'),imresize(K,sc,'bilinear'),0.8,1.3,1700,2500,0.1);
elseif ImQuery==0
    H = overlim4(JJ,label2rgb(imresize(bw_label,sc,'nearest'),'jet',[0,0,0],'shuffle'),imresize(K,sc,'bilinear'),0.001,1.001,1700,2500,0.1);
end
        

function [L,L2] = track2(J,K,D2,vx,~,~,bw_label,bitdepth,W,exmax,sc,st)
      
    % Track processing
for k = 1:max(bw_label(:))
	L = (K-D2).*imdilate(bw_label==k, strel('disk',1));
	L2 = (J-D2).*imdilate(bw_label==k, strel('disk',1));
    [veto,j] = max(L2(:));
    x = floor((j-1)/vx)+1;
    y = (j-1)-vx*(x-1)+1;

        if (veto<0.75*(2^bitdepth(1)))

            % Track extent
            extt = extrack(L,x,y,W);
            
            % Fourier analysis
            Z1 = sum(abs(fft(L,[],1)),2);
            Z2 = sum(abs(fft(L,[],2)),1);

            z1 = find(Z1<(max(Z1(:))+min(Z1(:)))/2, 1, 'first');
            z2 = find(Z2<(max(Z2(:))+min(Z2(:)))/2, 1, 'first');

            if (max(z1,z2)>st && extt<=exmax)
                set(text(sc*x-W,sc*y+W,[int2str(k) ' / ' num2str(max(z1,z2)) ' / ' num2str(extt) ]),'Color','w');
            else
                set(text(sc*x-W,sc*y+W,[int2str(k) ' / ' num2str(max(z1,z2)) ' / ' num2str(extt) ]),'Color','r');
            end
        end
end
    
    
function [K, bw_label] = trklbl(J,HT_val,mask_em_val,GauPar)

K=imfilter(J,fspecial('Gaussian',[21 21],GauPar),'replicate');
HT = imhmax(K,HT_val);  % this needs to be optimized 300

mask_em = HT>mask_em_val;
mask_em = imfill(mask_em,'holes');
mask_em = bwareaopen(mask_em, 5);

bw_label = bwlabel(mask_em,8);
